/*
 * Copyright (c) 1995, 2008, Oracle and/or its affiliates. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Oracle or the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.erubioman;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.SwingUtilities;

import com.erubioman.CircParserWrapper;

public class CircParserUi extends JPanel implements ActionListener, Loggable {
	/**
	 * 
	 */
	private class Execucio implements Runnable {
		private CircParserUi parent;

		Execucio(CircParserUi _parent) {
			parent = _parent;
		}

		public void run() {
			// Turn off metal's use of bold fonts
			UIManager.put("swing.boldMetal", Boolean.FALSE);
			parent.createAndShowGUI();
		}

	};

	private static final long serialVersionUID = 1L;
	JButton openLogisimButton, openVerilUocButton, openGuessTypeButton;
	JTextArea log;
	JFileChooser fc;

	public CircParserUi() {
		super(new BorderLayout());

		// Create the log first, because the action listeners
		// need to refer to it.
		log = new JTextArea(20, 80);
		log.setMargin(new Insets(5, 5, 5, 5));
		log.setEditable(false);
		JScrollPane logScrollPane = new JScrollPane(log);

		// Create a file chooser
		fc = new JFileChooser();

		openLogisimButton = new JButton("Obrir CIRC Logisim...");
		openLogisimButton.addActionListener(this);

		openVerilUocButton = new JButton("Obrir CIRC VerilUOC...");
		openVerilUocButton.addActionListener(this);

		openGuessTypeButton = new JButton("Obrir CIRC desconegut...");
		openGuessTypeButton.addActionListener(this);

		
		// For layout purposes, put the buttons in a separate panel
		JPanel buttonPanel = new JPanel(); // use FlowLayout
		buttonPanel.add(openLogisimButton);
		buttonPanel.add(openVerilUocButton);
		buttonPanel.add(openGuessTypeButton);

		// Add the buttons and the log to this panel.
		add(buttonPanel, BorderLayout.PAGE_START);
		add(logScrollPane, BorderLayout.CENTER);
	}

	public void actionPerformed(ActionEvent e) {

		// Handle open button action.
		int fileType = e.getSource() == openVerilUocButton ? 1 : e.getSource() == openGuessTypeButton ? -1 : 0;
		if ((fileType != 0) || (e.getSource() == openLogisimButton)) {
			boolean done = false;

			fc.setDialogTitle("Seleccionar fitxer CIRC");
			if (fc.showOpenDialog(CircParserUi.this) == JFileChooser.APPROVE_OPTION) {
				File inFile = fc.getSelectedFile();
				fc.setDialogTitle("Seleccionar fitxer Verilog de destinació");
				if (fc.showSaveDialog(CircParserUi.this) == JFileChooser.APPROVE_OPTION) {
					done = true;
					new CircParserWrapper(this).Parse(inFile, fc.getSelectedFile(), fileType);
				}
			}
			if (!done) {
				addLog("Operació cancel·lada per l'usuari.\n");
			}
		}
	}

	/**
	 * Create the GUI and show it. For thread safety, this method should be
	 * invoked from the event dispatch thread.
	 */
	private void createAndShowGUI() {
		// Create and set up the window.
		JFrame frame = new JFrame("Conversor CIRC a Verilog");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// Add content to the window.
		frame.add(new CircParserUi());

		// Display the window.
		frame.pack();
		frame.setVisible(true);
	}

	public void execute() {
		// Schedule a job for the event dispatch thread:
		// creating and showing this application's GUI.
		SwingUtilities.invokeLater(new Execucio(this));
	}

	public void addLog(String text) {
		log.append(text);
		log.setCaretPosition(log.getDocument().getLength());
	}
}
