/*
 * Copyright (c) 1995, 2008, Oracle and/or its affiliates. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Oracle or the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package com.erubioman.verilog.metrics;

import java.io.*;
import java.util.ArrayList;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import com.erubioman.Loggable;

public class UiLaunch extends JPanel implements ActionListener, Loggable {

	private static final long serialVersionUID = 1L;
	JButton openButton;
	JTextArea log;
	JFileChooser fc;
	boolean busy;

	private UiLaunch() {
		super(new BorderLayout());

		// Create the log first, because the action listeners
		// need to refer to it.
		log = new JTextArea(20, 80);
		log.setMargin(new Insets(5, 5, 5, 5));
		log.setEditable(false);
		JScrollPane logScrollPane = new JScrollPane(log);

		// Create a file chooser
		busy = false;
		fc = new JFileChooser();

		openButton = new JButton("Iniciar Anàlisi ...", null);
		openButton.addActionListener(this);

		// For layout purposes, put the buttons in a separate panel
		JPanel buttonPanel = new JPanel(); // use FlowLayout
		buttonPanel.add(openButton);

		// Add the buttons and the log to this panel.
		add(buttonPanel, BorderLayout.PAGE_START);
		add(logScrollPane, BorderLayout.CENTER);
	}

	public void actionPerformed(ActionEvent e) {

		// Handle open button action.
		if ((e.getSource() == openButton) && !busy) {
			boolean done = false;

			fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
			fc.setDialogTitle("Escollir fitxer CSV de descripció d'exercicis (opcional)");
			File csv = fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION ? fc.getSelectedFile() : null;
			if (csv != null) {
				log.append("Obrint fitxer CSV: " + csv.getAbsolutePath() + "\n");
			}
			fc.setDialogTitle("Escollir fitxer CSV de resultats");
			if (fc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
				File outFile = fc.getSelectedFile();
				log.append("Fitxer CSV de resultats: " + outFile.getAbsolutePath() + "\n");
				fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				fc.setDialogTitle("Escollir directori(s) d'exercicis. Cancel.lar per finalitzar");
				ArrayList<String> exercicis = new ArrayList<String>();
				while (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
					String directory = fc.getSelectedFile().toString();
					log.append("Afegit directori d'exercicis: " + directory + "\n");
					exercicis.add(directory);
				}
				if (exercicis.size() > 0) {
					done = true;
					Loggable logDestination = this;
					SwingWorker<Integer, Void> worker = new SwingWorker<Integer, Void>() {
						@Override
						public Integer doInBackground() {
							busy = true;
							return new VerilCircMetrics(logDestination).analyzeFiles(csv,
									exercicis.toArray(new String[exercicis.size()]), outFile, null);
						}

						@Override
						public void done() {
							busy = false;
						}
					};
					worker.execute();
				}
			}
			if (!done) {
				addLog("Operació cancel·lada per l'usuari.\n");
			}
		}
	}

	/**
	 * Create the GUI and show it. For thread safety, this method should be
	 * invoked from the event dispatch thread.
	 */
	private static void mostraUI() {
		// Create and set up the window.
		JFrame frame = new JFrame("VerilCircMetrics");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// Add content to the window.
		frame.add(new UiLaunch());

		// Display the window.
		frame.pack();
		frame.setVisible(true);
	}

	public static void execute() {
		// Schedule a job for the event dispatch thread:
		// creating and showing this application's GUI.
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				// Turn off metal's use of bold fonts
				UIManager.put("swing.boldMetal", Boolean.FALSE);
				mostraUI();
			}
		});
	}

	public void addLog(String text) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				log.append(text);
				log.setCaretPosition(log.getDocument().getLength());
			}
		});
	}
}
